@@ -5,24 +5,23 @@ use strict;
 use warnings;
 
 our $VERSION;
-$VERSION = sprintf "%d.%02d", q$Name: Release-0-28 $ =~ /Release-(\d+)-(\d+)/;
+$VERSION = sprintf "%d.%02d", q$Name: Release-1-00 $ =~ /Release-(\d+)-(\d+)/;
+#$VERSION = sprintf "%s", q$Name: Release-v0.304.0$ =~ /Release-(v\d+\.\d+\.\d+)/;
+#our $NVERSION;			# pure numeric 2-part equivalent version
+#($NVERSION = $VERSION) =~ s/v(\d+\.\d+)\.\d+/$1/;
 
 require Exporter;
 our @ISA = qw(Exporter);
 
-our @EXPORT = qw(
+our @EXPORT = qw();
+our @EXPORT_OK = qw(
 	id2ppath ppath2id s2ppchars id2pairpath pairpath2id
-	pt_lsnode pt_lstree pt_mknode pt_mktree pt_rmnode
-	pt_mkbud
-	up_dir get_prefix
+	pt_lsid pt_lstree pt_mkid pt_mktree pt_rmid
+	pt_budstr get_prefix
 	$pfixtail
 	$pair $pairp1 $pairm1
 );
-# XXXX exporting way to much?
-
-our @EXPORT_OK = qw(
-	$pair $pairp1 $pairm1
-);
+our %EXPORT_TAGS = (all => [ @EXPORT_OK ]);
 
 our @EXPORT_FAIL = qw(
 	pair=1 pair=2 pair=3 pair=4 pair=5 pair=6 pair=7 pair=8 pair=9
@@ -87,22 +86,6 @@ my $P = $proper_ppath_re;
 # $pt_objid =~ tr/\/:./=+,/;
 # my $pt_prefix = $namespace."/pairtree_root/".join('/', $pt_objid =~ /..|.$/g);
 
-# ---------
-# Copyright 2008-2009 Regents of the University of California
-# 
-# Licensed under the Apache License, Version 2.0 (the "License"); you may
-# not use this file except in compliance with the License. You may obtain a
-# copy of the License at
-# 
-#         http://www.apache.org/licenses/LICENSE-2.0
-# 
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
-# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
-# License for the specific language governing permissions and limitations
-# under the License. 
-# ---------
-
 # id2ppath - return /-terminated ppath corresponding to id
 # 
 # For Perl, the platform's path component separator ('/' or '\') is
@@ -252,23 +235,15 @@ sub ppath2id{ my( $path, $pathcomp_sep )=@_;	# single arg form, second
 }
 
 use Carp;
+use File::Spec;
 use File::Find;
 use File::Path;
-use File::Value;
-use File::ANVL;
+use File::Namaste qw( nam_add );
+use File::Value ':all';
 use File::Glob ':glob';		# standard use of module, which we need
 				# as vanilla glob won't match whitespace
 
-# Return parent with trailing slash intact.
-#
-sub up_dir { ( $_ )=@_;
-
-	return "/"		if m,^/+$,;
-	return "./"		if m,^\./*$,;
-	s,[^/]+/*$,,;
-	return "./"		if m,^$,;
-	return $_;
-}
+our $Win;			# whether we're running on Windows
 
 my $pfixtail = 'pairtree_prefix';
 
@@ -288,15 +263,16 @@ sub get_prefix { my( $parent_dir )=@_;
 # return 0 on success, 1 on soft fail, >1 on hard fail
 # xxxxxxxxx get consistent on these return codes/croaks
 #
-sub pt_lsnode { my( $dir, $id, $r_opt )=@_;
+sub pt_lsid { my( $dir, $id, $r_opt )=@_;
 
 	$dir		or croak "no dir or empty dir";
 	$id		or croak "no id or empty id";
 	ref($r_opt) eq "HASH" or
 		croak "r_opt must reference a hash (for input/output)";
 
+	$dir = fiso_dname($dir, $R);	# make sure we have descender
 	my $parent_dir = $$r_opt{parent_dir}
-		|| up_dir($dir);
+		|| fiso_uname($dir);
 	my $prefix = $$r_opt{prefix}
 		|| get_prefix($parent_dir);
 
@@ -343,6 +319,178 @@ sub pt_lsnode { my( $dir, $id, $r_opt )=@_;
 	return 0;
 }
 
+my $homily = "(pairpath end should be followed by only one thing -- " .
+		"a directory name more than $pair characters long)";
+
+# Create a closure to hold a stateful node-visiting subroutine and other
+# options suitable to be passed as the options parameter to File::Find.
+# Returns the small hash { 'wanted' => $visitor, 'follow' => 1 } and a
+# subroutine $visit_over that can be called to summarize the visit.
+#
+sub make_visitor { my( $r_opt )=@_;
+
+	my $om = $r_opt->{om} or
+		return undef;
+
+	my $objectcount = 0;		# number of objects encountered
+	my $filecount = 0;		# number of files encountered xxx
+	my $dircount = 0;		# number of directories encountered xxx
+	my $symlinkcount = 0;		# number of symlinks encountered xxx
+	my $irregularcount = 0;		# non file, non dir fs items to report
+
+	my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $sze);
+	my ($pdname, $tpname, $wpname);
+	my $symlinks_followed = 1;
+
+	# yyy do as $curobj = {...}; ?
+	my %curobj = (
+		'ppath' => '',
+		'encaperr' => 0,
+		'octets' => 0,
+		'streams' => 0,
+	);
+
+    # xxx move this outside closure and give it a $currobj arg?
+    #     or is this much more efficient as-is?
+    # xxx nail down everything that won't change during lstree
+    my $newobj = sub { my( $ppath, $encaperr, $octets, $streams )=@_;
+
+	# warning: ugly code ahead
+	if ($curobj{'ppath'}) {		# print record of previous obj
+		$_ = ppath2id($curobj{'ppath'});
+		s/^/$r_opt->{prefix}/;		# uses global set in lstree()
+		$r_opt->{long} and
+			$om->elem('node',
+				join("  ", $_, $curobj{'ppath'},
+				"$curobj{'octets'}.$curobj{'streams'}")), 1
+		or
+			$om->elem('node', $_), 1
+		;
+		$curobj{'ppath'} eq $ppath and
+			print "error: corrupted pairtree at pairpath ",
+				"$ppath/: split end $homily\n";
+		# xxx use om?
+	}
+	# xxx strange
+	die "newobj: all args must be defined"
+		unless (defined($ppath) && defined($encaperr)
+			&& defined($octets) && defined($streams));
+	$curobj{'ppath'} = $ppath;
+	$curobj{'encaperr'} = $encaperr;
+	$curobj{'octets'} = $octets;
+	$curobj{'streams'} = $streams;
+    };
+
+    my $visitor = sub {		# receives no args from File::Find
+
+	$pdname = $File::Find::dir;		# current parent directory name
+	$tpname = $_;				# current filename in that dir
+	$_ = $wpname = $File::Find::name;	# whole pathname to file
+
+	# We always need lstat() info on the current node XXX why?
+	# xxx tells us all, but if following symlinks the lstat is done
+	# ... by find:  use (-X _), but of the nifty facts below we
+	# still need to harvest the size ($sze) by hand.
+	#
+	($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $sze) = lstat($tpname)
+		unless ($symlinks_followed and ($sze = -s _));
+
+	#print "NEXT: $pdname $_ $wpname\n";
+
+	# If we follow symlinks (usual), we have to expect the -l type,
+	# which hides the type of the link target (what we really want).
+	#
+	if (! $Win and -l _) {
+		$symlinkcount++;
+		#print "XXXX SYMLINK $_\n";
+		# yyy presumably this branch never happens when
+		#     _not_ following links?
+		($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $sze)
+			= stat($tpname);	# get the real thing
+	}
+	# After this, tests of the form (-X _) give almost everything.
+
+	if (-f $tpname) {
+		$filecount++;
+		if (m@^.*$R/(.*/)?pairtree.*$@o) {
+			### print "$pdname PTREEFILE $tpname\n";
+			# xxx    if $verbose;
+			# else -prune ??
+		}
+		elsif (m@^.*$R/$P/[^/]+$@o) {
+			#print "m@.*$R/$P/[^/]+@: $_\n";
+		 	#print "$pdname UF $tpname\n";
+			print "error: corrupted pairtree at pairpath ",
+				"$pdname/: found unencapsulated file ",
+				"'$tpname' $homily\n";
+		}
+		else {
+			# xxx sanity check that $curobj is defined
+			$curobj{'octets'} += $sze;
+			### print "sssss $curobj{'octets'}\n";
+			$curobj{'streams'}++;
+		#	-fprintf $altout 'IN %p %s\n'
+		#	$noprune
+		}
+	}
+	elsif (-d $tpname) {
+		$dircount++;
+		if (m@^.*$R/(.*/)?pairtree.*$@o) {
+			#print "$pdname PTREEDIR $tpname\n";
+			# xxx if $verbose;
+		#	-prune
+		}
+		# At last, we're entering a "regular" object.
+		elsif (m@^.*$R/($P/)?[^/]{$pairp1,}$@o) {
+			# start new object; but end previous object first
+			# form: ppath, EncapErr, octets, streams
+			$objectcount++;
+			&$newobj($pdname, 0, 0, 0);
+			# print "$pdname NS $tpname\n";
+			#	-fprintf $altout 'START %h 0\n'
+			#	$noprune
+		}
+		elsif (m@^.*$R/$P$@o) {
+			#	-empty
+			# xxx if $verbose...	-printf '%p EP -\n'
+		}
+		# $pair, $pairm1, $pairp1
+		# We have a post-morty encapsulation error
+		elsif (m@^.*$R/([^/]{$pair}/)*[^/]{1,$pairm1}/[^/]{1,$pair}$@o) {
+			#print "$pdname PM $tpname\n";
+			print "error: corrupted pairtree at pairpath ",
+				"$pdname/: found '$tpname' after forced ",
+				"path ending $homily\n";
+				
+			#	-fprintf $altout 'START %h 0\n'
+			#	$noprune
+		}
+	}
+	else {
+		$irregularcount++;
+	}
+    };
+
+    my $visit_over = sub { my( $ret, $tree )=@_;
+
+	$ret ||= 0;
+
+	# Dummy call to pt_newobj() to cough up the last buffered object.
+	# xxx not multi-threadable!  how to give newobj its own context?
+	&$newobj("", 0, 0, 0);			# shake out the last one
+
+	# XXX what does find return?
+	$om->elem("lstree", "find returned '$ret' for $tree")	if $ret;
+	$om->elem("objectcount", "$objectcount object" .
+		($objectcount == 1 ? "" : "s"));
+
+	return ($objectcount);
+    };
+
+    	return ({ 'wanted' => $visitor, 'follow' => $r_opt->{follow_fast} },
+		$visit_over);
+}
+
 my $objectcount = 0;		# number of objects encountered
 my $filecount = 0;		# number of files encountered xxx
 my $dircount = 0;		# number of directories encountered xxx
@@ -361,35 +509,48 @@ sub pt_lstree { my( $tree, $r_opt, $r_visit_node, $r_wrapup )=@_;
 	#ref( $r_wrapup ||= \&pt_lstree_wrapup ) eq "CODE" or
 	#	croak "r_wrapup must reference a node-visiting function";
 
-	$$r_opt{parent_dir} ||= up_dir($tree);
+	# xxx should probably test directly for symlink capacity
+	defined($Win) or	# if we're on a Windows platform avoid -l
+		$Win = grep(/Win32|OS2/i, @File::Spec::ISA);
+
+	$tree = fiso_dname($tree, $R);	# make sure we have descender
+	$$r_opt{parent_dir} ||= fiso_uname($tree);
 	$$r_opt{prefix} ||= get_prefix($$r_opt{prefix});
 
-	$gr_opt = $r_opt;	# make options available to find
-	my %find_opt = (
-		'wanted'	=>  $r_visit_node,
-		'follow_fast'	=>  $$r_opt{follow_fast},
-	);
+	my ($find_opt, $visit_over) = make_visitor($r_opt);
+	$find_opt or
+		croak "make_visitor() failed";
+	my $ret = find($find_opt, $tree);
+	$visit_over and
+		&$visit_over($ret, $tree);
+	return $ret;
 
-	#print "Id      Oxum\n";		# XXXXXX this is our r_startup
-
-	my $ret = find(\%find_opt, $tree);
-
-	# XXXXX this is our r_wrapup
-	# Dummy call to pt_newobj() to cough up the last buffered object.
-	pt_newobj("", 0, 0, 0);			# shake out the last one
-	# XXX what does find return?
-	#print "lstree: find returned '$ret' for $tree"		if $ret;
-	$gr_opt->{om}->elem('lstree', "find returned '$ret' for $tree")	if $ret;
-	#print "$objectcount object", ($objectcount == 1 ? "" : "s"), "\n";
-	$gr_opt->{om}->elem('objectcount', "$objectcount object" .
-		($objectcount == 1 ? "" : "s"));
+#	$gr_opt = $r_opt;	# make options available to find
+#	my %find_opt = (
+#		'wanted'	=>  $r_visit_node,
+#		'follow_fast'	=>  $$r_opt{follow_fast},
+#	);
+#
+#	#print "Id      Oxum\n";		# XXXXXX this is our r_startup
+#
+#	my $ret = find(\%find_opt, $tree);
+#
+#	# XXXXX this is our r_wrapup
+#	# Dummy call to pt_newobj() to cough up the last buffered object.
+#	pt_newobj("", 0, 0, 0);			# shake out the last one
+#	# XXX what does find return?
+#	#print "lstree: find returned '$ret' for $tree"		if $ret;
+#	$gr_opt->{om}->elem('lstree', "find returned '$ret' for $tree")	if $ret;
+#	#print "$objectcount object", ($objectcount == 1 ? "" : "s"), "\n";
+#	$gr_opt->{om}->elem('objectcount', "$objectcount object" .
+#		($objectcount == 1 ? "" : "s"));
 
 	return $ret;
 }
 
-# Create the bud that will encapsulate the leaf node
+# Create the bud string that will encapsulate the leaf node
 #
-sub pt_mkbud { my( $id, $bud_style )=@_;
+sub pt_budstr { my( $id, $bud_style )=@_;
 
 	# Xxx add chars if less than $pair chars in $_
 	$id ||= "";
@@ -409,15 +570,16 @@ sub pt_mkbud { my( $id, $bud_style )=@_;
 	return s2ppchars($id);
 }
 
-sub pt_mknode { my( $dir, $id, $r_opt )=@_;
+sub pt_mkid { my( $dir, $id, $r_opt )=@_;
 
 	$dir		or croak "no dir or empty dir";
 	$id		or croak "no id or empty id";
 	ref($r_opt) eq "HASH" or
 		croak "r_opt must reference a hash (for input/output)";
 
+	$dir = fiso_dname($dir, $R);	# make sure we have descender
 	my $parent_dir = $$r_opt{parent_dir}
-		|| up_dir($dir);
+		|| fiso_uname($dir);
 	my $prefix = $$r_opt{prefix}
 		|| get_prefix($parent_dir);
 
@@ -430,16 +592,16 @@ sub pt_mknode { my( $dir, $id, $r_opt )=@_;
 
 	-d $dir or			# need to create base directory
 		pt_mktree($dir, "", $r_opt) and		# if error
-		($$r_opt{msg} = "pt_mknode: $$r_opt{msg}"),
+		($$r_opt{msg} = "pt_mkid: $$r_opt{msg}"),
 		return 1;		# return after adding our stamp
 	my $ppath = $parent_dir . id2ppath($id);
-	my $bud = $ppath . pt_mkbud($id, $$r_opt{bud_style});
+	my $bud = $ppath . pt_budstr($id, $$r_opt{bud_style});
 
 	my $ret;
 	eval { $ret = mkpath($bud) };
 	$@		and croak "Couldn't create $bud: $@";
 	if ($ret == 0) {
-		croak "pt_mknode: mkpath returned '0' for $bud"
+		croak "pt_mkid: mkpath returned '0' for $bud"
 			unless -e $bud;
 		$$r_opt{msg} = "error: $bud ($id) already exists\n";
 		return 0;
@@ -450,8 +612,6 @@ sub pt_mknode { my( $dir, $id, $r_opt )=@_;
 	return 1;
 }
 
-use File::Namaste;
-
 sub pt_mktree { my( $dir, $prefix, $r_opt )=@_;
 
 	# XXXX make up my mind about when to croak and when to
@@ -462,7 +622,8 @@ sub pt_mktree { my( $dir, $prefix, $r_opt )=@_;
 		croak "r_opt must reference a hash (for input/output)";
 			# except that we ignore any r_opt inputs here
 
-	my $parent_dir = up_dir($dir);
+	$dir = fiso_dname($dir, $R);	# make sure we have descender
+	my $parent_dir = fiso_uname($dir);
 	my $ret;
 	eval { $ret = mkpath($dir) };
 	if ($@) {
@@ -482,22 +643,26 @@ sub pt_mktree { my( $dir, $prefix, $r_opt )=@_;
 		$$r_opt{msg} = "$pxfile: $msg";
 		return 1;
 	}
-	$msg = nam_set($dir, undef, '0', "pairtree_$VERSION");
-	# xxxx croak or return via r_opt{msg}
 	$msg		and croak "Couldn't create namaste tag in $dir: $msg";
+	$msg = nam_add($parent_dir, undef, '0', "pairtree_$VERSION", 0);
+		# yyy better to use 0 to mean "don't truncate"
+		#length("pairtree_$VERSION"));
+	# xxxx croak or return via r_opt{msg}
+	$msg	and croak "Couldn't create namaste tag in $parent_dir: $msg";
 
 	return 0;
 }
 
-sub pt_rmnode { my( $dir, $id, $r_opt )=@_;
+sub pt_rmid { my( $dir, $id, $r_opt )=@_;
 
 	$dir		or croak "no dir or empty dir";
 	$id		or croak "no id or empty id";
 	ref($r_opt) eq "HASH" or
 		croak "r_opt must reference a hash (for input/output)";
 
+	$dir = fiso_dname($dir, $R);	# make sure we have descender
 	my $parent_dir = $$r_opt{parent_dir}
-		|| up_dir($dir);
+		|| fiso_uname($dir);
 	my $prefix = $$r_opt{prefix}
 		|| get_prefix($parent_dir);
 
@@ -522,7 +687,7 @@ sub pt_rmnode { my( $dir, $id, $r_opt )=@_;
 	my $ret;
 	eval { $ret = rmtree($ppath) };
 	if ($@) {
-		$$r_opt{msg} = "Couldn't create $ppath tree: $@";
+		$$r_opt{msg} = "Couldn't remove $ppath tree: $@";
 		return 1;
 	}
 	if ($ret == 0) {
@@ -555,9 +720,6 @@ my %curobj = (
 	'streams' => 0,
 );
 
-my $homily = "(pairpath end should be followed by only one thing -- " .
-		"a directory name more than $pair characters long)";
-
 sub pt_newobj { my( $ppath, $encaperr, $octets, $streams )=@_;
 
 	# warning: ugly code ahead
@@ -605,7 +767,7 @@ sub pt_visit_node {	# receives no args
 	# If we follow symlinks (usual), we have to expect the -l type,
 	# which hides the type of the link target (what we really want).
 	#
-	if (-l _) {
+	if (! $Win and -l _) {
 		$symlinkcount++;
 		print "XXXX SYMLINK $_\n";
 		# yyy presumably this branch never happens when
@@ -695,11 +857,11 @@ File::Pairtree - routines to manage pairtrees
  ppath2id($path);              # returns id corresponding to $path
  ppath2id($path, $separator);  # if you want an alternate separator char
 
- pt_mkbud();
- pt_mknode();
+ pt_budstr();
+ pt_mkid();
  pt_mktree();
- pt_rmnode();
- pt_lsnode();
+ pt_rmid();
+ pt_lsid();
 
 =head1 DESCRIPTION
 
@@ -707,7 +869,7 @@ This is very brief documentation for the B<Pairtree> Perl module.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2008-2010 UC Regents.  Open source BSD license.
+Copyright 2008-2011 UC Regents.  Open source BSD license.
 
 =cut
 
@@ -950,7 +1112,7 @@ sub visit{	# receives no args
 	# If we get here, the stack top's pp is the same as our parent
 	# (or it's empty because we're at our first node ever).
 	#
-	if (-l _) {
+	if (! $Win and -l _) {
 		print "XXXX SYMLINK $_\n";
 		# XXX what does this branch do when _not_ following links?
 		($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $sze)
